# 1 "/builddir/build/BUILD/cp2k-2023.1/src/fm/cp_fm_vect.F"
!--------------------------------------------------------------------------------------------------!
!   CP2K: A general program to perform molecular dynamics simulations                              !
!   Copyright 2000-2022 CP2K developers group <https://cp2k.org>                                   !
!                                                                                                  !
!   SPDX-License-Identifier: GPL-2.0-or-later                                                      !
!--------------------------------------------------------------------------------------------------!

! **************************************************************************************************
!> \brief routine to handle vectors of full matrixes
!> \par History
!>      08.2002 created [fawzi]
!> \author Fawzi Mohamed
! **************************************************************************************************
MODULE cp_fm_vect
   USE cp_fm_types, ONLY: cp_fm_p_type, &
                          cp_fm_release, cp_fm_type
#include "../base/base_uses.f90"

# 23 "/builddir/build/BUILD/cp2k-2023.1/src/fm/cp_fm_vect.F"

   IMPLICIT NONE
   PRIVATE

   LOGICAL, PRIVATE, PARAMETER :: debug_this_module = .TRUE.
   CHARACTER(len=*), PARAMETER, PRIVATE :: moduleN = 'cp_fm_vect'

   PUBLIC :: cp_fm_vect_dealloc

   INTERFACE cp_fm_vect_dealloc
# 34 "/builddir/build/BUILD/cp2k-2023.1/src/fm/cp_fm_vect.F"
# 35 "/builddir/build/BUILD/cp2k-2023.1/src/fm/cp_fm_vect.F"
            MODULE PROCEDURE cp_fm_m1_dealloc_array_alloc
# 35 "/builddir/build/BUILD/cp2k-2023.1/src/fm/cp_fm_vect.F"
            MODULE PROCEDURE cp_fm_m1_dealloc_array_pointer
# 37 "/builddir/build/BUILD/cp2k-2023.1/src/fm/cp_fm_vect.F"
# 34 "/builddir/build/BUILD/cp2k-2023.1/src/fm/cp_fm_vect.F"
# 35 "/builddir/build/BUILD/cp2k-2023.1/src/fm/cp_fm_vect.F"
            MODULE PROCEDURE cp_fm_m1_dealloc_p_type_alloc
# 35 "/builddir/build/BUILD/cp2k-2023.1/src/fm/cp_fm_vect.F"
            MODULE PROCEDURE cp_fm_m1_dealloc_p_type_pointer
# 37 "/builddir/build/BUILD/cp2k-2023.1/src/fm/cp_fm_vect.F"
# 38 "/builddir/build/BUILD/cp2k-2023.1/src/fm/cp_fm_vect.F"
   END INTERFACE
!***
CONTAINS

# 43 "/builddir/build/BUILD/cp2k-2023.1/src/fm/cp_fm_vect.F"
# 44 "/builddir/build/BUILD/cp2k-2023.1/src/fm/cp_fm_vect.F"
! **************************************************************************************************
!> \brief deallocate an array of pointers to blacs matrixes
!> \param matrices the array of matrices to deallocate
!> \par History
!>      07.2002 created [fawzi]
!> \author Fawzi Mohamed
! **************************************************************************************************
         SUBROUTINE cp_fm_m1_dealloc_array_alloc (matrices)
            TYPE(cp_fm_type), DIMENSION(:), ALLOCATABLE          :: matrices

            INTEGER                                            :: i

            IF (ALLOCATED (matrices)) THEN
               DO i = 1, SIZE(matrices)
                  CALL cp_fm_release(matrices(i) )
# 62 "/builddir/build/BUILD/cp2k-2023.1/src/fm/cp_fm_vect.F"
               END DO
               DEALLOCATE (matrices)
# 67 "/builddir/build/BUILD/cp2k-2023.1/src/fm/cp_fm_vect.F"
            END IF
         END SUBROUTINE cp_fm_m1_dealloc_array_alloc
# 44 "/builddir/build/BUILD/cp2k-2023.1/src/fm/cp_fm_vect.F"
! **************************************************************************************************
!> \brief deallocate an array of pointers to blacs matrixes
!> \param matrices the array of matrices to deallocate
!> \par History
!>      07.2002 created [fawzi]
!> \author Fawzi Mohamed
! **************************************************************************************************
         SUBROUTINE cp_fm_m1_dealloc_array_pointer (matrices)
            TYPE(cp_fm_type), DIMENSION(:), POINTER          :: matrices

            INTEGER                                            :: i

            IF (ASSOCIATED (matrices)) THEN
               DO i = 1, SIZE(matrices)
                  CALL cp_fm_release(matrices(i) )
# 62 "/builddir/build/BUILD/cp2k-2023.1/src/fm/cp_fm_vect.F"
               END DO
               DEALLOCATE (matrices)
# 65 "/builddir/build/BUILD/cp2k-2023.1/src/fm/cp_fm_vect.F"
                  NULLIFY (matrices)
# 67 "/builddir/build/BUILD/cp2k-2023.1/src/fm/cp_fm_vect.F"
            END IF
         END SUBROUTINE cp_fm_m1_dealloc_array_pointer
# 70 "/builddir/build/BUILD/cp2k-2023.1/src/fm/cp_fm_vect.F"
# 43 "/builddir/build/BUILD/cp2k-2023.1/src/fm/cp_fm_vect.F"
# 44 "/builddir/build/BUILD/cp2k-2023.1/src/fm/cp_fm_vect.F"
! **************************************************************************************************
!> \brief deallocate an array of pointers to blacs matrixes
!> \param matrices the array of matrices to deallocate
!> \par History
!>      07.2002 created [fawzi]
!> \author Fawzi Mohamed
! **************************************************************************************************
         SUBROUTINE cp_fm_m1_dealloc_p_type_alloc (matrices)
            TYPE(cp_fm_p_type), DIMENSION(:), ALLOCATABLE          :: matrices

            INTEGER                                            :: i

            IF (ALLOCATED (matrices)) THEN
               DO i = 1, SIZE(matrices)
                  CALL cp_fm_release(matrices(i) %matrix)
# 60 "/builddir/build/BUILD/cp2k-2023.1/src/fm/cp_fm_vect.F"
                     DEALLOCATE (matrices(i)%matrix)
# 62 "/builddir/build/BUILD/cp2k-2023.1/src/fm/cp_fm_vect.F"
               END DO
               DEALLOCATE (matrices)
# 67 "/builddir/build/BUILD/cp2k-2023.1/src/fm/cp_fm_vect.F"
            END IF
         END SUBROUTINE cp_fm_m1_dealloc_p_type_alloc
# 44 "/builddir/build/BUILD/cp2k-2023.1/src/fm/cp_fm_vect.F"
! **************************************************************************************************
!> \brief deallocate an array of pointers to blacs matrixes
!> \param matrices the array of matrices to deallocate
!> \par History
!>      07.2002 created [fawzi]
!> \author Fawzi Mohamed
! **************************************************************************************************
         SUBROUTINE cp_fm_m1_dealloc_p_type_pointer (matrices)
            TYPE(cp_fm_p_type), DIMENSION(:), POINTER          :: matrices

            INTEGER                                            :: i

            IF (ASSOCIATED (matrices)) THEN
               DO i = 1, SIZE(matrices)
                  CALL cp_fm_release(matrices(i) %matrix)
# 60 "/builddir/build/BUILD/cp2k-2023.1/src/fm/cp_fm_vect.F"
                     DEALLOCATE (matrices(i)%matrix)
# 62 "/builddir/build/BUILD/cp2k-2023.1/src/fm/cp_fm_vect.F"
               END DO
               DEALLOCATE (matrices)
# 65 "/builddir/build/BUILD/cp2k-2023.1/src/fm/cp_fm_vect.F"
                  NULLIFY (matrices)
# 67 "/builddir/build/BUILD/cp2k-2023.1/src/fm/cp_fm_vect.F"
            END IF
         END SUBROUTINE cp_fm_m1_dealloc_p_type_pointer
# 70 "/builddir/build/BUILD/cp2k-2023.1/src/fm/cp_fm_vect.F"
# 71 "/builddir/build/BUILD/cp2k-2023.1/src/fm/cp_fm_vect.F"

END MODULE cp_fm_vect
